#!/usr/bin/perl -w

#
# Jdb::IO::Writer.pm
# $Id: Writer.pm 522 2007-11-11 05:45:40Z johnh $
#
# Copyright (C) 2005-2007 by John Heidemann <johnh@isi.edu>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2, as published by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
#


package Jdb::IO::Writer;

=head1 NAME

Jdb::IO::Writer - handle formatting reading from a jdb file

=cut

@ISA = qw(Jdb::IO);
($VERSION) = ('$Revision: 522 $' =~ m/(\d+)/);

use strict;
use IO::File;
use Carp;
use Jdb::IO;

=head2 new

    $jdb = new Jdb::IO::Writer(-file => $filename);
    $jdb = new Jdb::IO::Writer(-header => "#h -Ft foo bar", -fh => $file_handle);

Creates a new writer object.
Always succeeds, but 
check the C<error> method to test for failure.

Options:

=over

=item -fh FILE_HANDLE
Write IO to the given file handle.

=item -file FILENAME
Create the given filename.

=item -header HEADER_LINE
Force the header to the given HEADER_LINE
=back

=item -outputheader [0|1]
If VALUE is 0, then
don't write the header (output will then not be jdb format).
(Defaults to 1.)

=back

=cut

sub new {
    my $class = shift @_;
    my $self = $class->SUPER::new(@_);
    bless $self, $class;
    #
    # new instance variables
    # (none)
    #
    $self->config(@_);
    #
    # setup:    
    if (!$self->{_fh}) {
	$self->{_error} = "cannot setup filehandle";
	return $self;
    };
    $self->{_outputheader} = 1 if (!defined($self->{_outputheader}));

    $self->write_headerrow;

    return $self;
}

=head2 config_one

documented in new

=cut
sub config_one {
    my($self, $aaref) = @_;
    if ($aaref->[0] eq '-file') {
	shift @$aaref;
	my($file) = shift @$aaref;
	my $fh = new IO::File $file, "w";
	if ($fh) {
	    $self->{_fh} = $fh;
	} else {
	    $self->{_error} = "cannot open $file";
	};
    } elsif ($aaref->[0] eq '-outputheader') {
	shift @$aaref;
	$self->{_outputheader} = shift @$aaref;
    } else {
	$self->SUPER::config_one($aaref);
    };
}

=head2 write_row_from_aref

    $jdb->write_row_from_aref(\@a);

Write @a.

=cut

sub write_row_from_aref {
    my($self, $aref) = @_;

    $self->write_headerrow unless ($self->{_header_set});
    $self->{_fh}->print(join($self->{_fs}, @$aref) . "\n");
}


=head2 write_row

    $jdb->write_row($a1, $a2...);

Write args out.  Less efficient than write_row_from_aref.

=cut

sub write_row {
    my($self) = shift @_;

    $self->write_row_from_aref(\@_);
}

=head2 write_row_from_href

    $jdb->write_row_from_aref(\%h);

Write out %h, a hash of the row fields where each key is a field name.

=cut

sub write_row_from_href {
    my($self, $href) = @_;

    my @a;
    foreach (@{$self->{_cols}}) {
	my $v = $href->{$_};
	push(@a, defined($v) ? $v : $self->{_empty});
    };
    $self->write_row_from_aref(\@a);
}

=head2 write_comment

    $jdb->write_comment($c);

Write out $c as a comment.
($c should be just the text, without a "# " header or a newline trailer.

=cut

sub write_comment {
    my($self, $c) = @_;

    $self->{_fh}->print("# " . $c . "\n");
}





=head2 write_headerrow

internal use only; write the header

=cut
sub write_headerrow {
    my($self) = @_;
    die "double header write.\n" if ($self->{_header_set});
    $self->{_fh}->print($self->{_headerrow} . "\n")
	if ($self->{_outputheader});
    $self->{_header_set} = 1;
};


#
# hack
#

=head2 format_jdb_fields

    format_jdb_fields(\%data, \@fields)

Returns a string representing double-space-separated, formatted version of
the hash'ed fields stoed in %data, listed in @fields.
(This routine is a hack, there needs to be a JdbWriter to do this properly,
but there isn't currently.

=cut

sub format_jdb_fields {
    my($data_href, $fields_aref) = @_;
    my $out = '';
    foreach (@$fields_aref) {
	my $val = defined($data_href->{$_}) ? $data_href->{$_} : '-';
	$val =~ s/\n/\\n/g;   # fix newlines
	$val =~ s/  +/ /g;   # fix double spaces
	$out .= $val . "  ";
    };
    $out =~ s/  $//;   # trim trailing spaces
    return $out;
}


1;
